@echo off

rem // iOS build environment CocoaPods daemon control script by Pierre-Marie Baty <pm@pmbaty.com>

setlocal enableextensions enabledelayedexpansion
goto startup

:usage
echo.
echo %SCRIPT_NAME% - Pierre-Marie Baty's cocoapods daemon for Windows
echo Part of the iOS Build Environment - https://www.pmbaty.com/iosbuildenv/
echo.
echo This daemon implements an almost Darwin-compatible BSD UNIX subsystem that
echo is able to run CocoaPods unmodified, without the crippling NTFS limitations
echo imposed by Microsoft on their operating systems.
echo.
echo Usage: cocoapods ^<start^|stop^|status^|command^>
echo.
echo    start: starts up the CocoaPods daemon ^(this can take up to one minute^)
echo    stop: shuts down a running CocoaPods daemon
echo    wait: waits until the CocoaPods daemon becomes ready
echo    status: reports the daemon status ^(ready, busy or stopped^)
echo.
echo    command: command to issue ^(e.g. "pod install", or "gem update cocoapods"^)
echo.
echo "cd" to where your Podfile is, then issue your CocoaPods command, e.g. run
echo "%SCRIPT_NAME% pod install".
echo                                          -- "Please, one miracle at a time."
goto exit_success

:startup

rem ////////////////////////////////////////
rem // CocoaPod daemon user-settable options

rem // minimum memory (in MB) necessary to run the daemon. Will refuse to start with an error message if this prerequisite is not met.
set MIN_MEMORY=3072

rem // maximum memory (in MB) that the daemon's virtual machine can reserve from the host system (i.e. your computer's operating system)
set MAX_MEMORY=8192

rem // if the daemon did not signal a successful start after this delay in seconds, the startup process will be aborted
set MAX_STARTUP_TIME_SECONDS=180

rem // if the daemon did not stop gracefully after this delay in seconds, the process will be aborted (killed)
set MAX_SHUTDOWN_TIME_SECONDS=30

rem // maximum delay in seconds a "cocoapods.cmd wait" operation is allowed to last. Such an operation returns success as soon as
rem // the daemon is ready to accept orders, or failure when the timeout expires, whichever is the soonest.
set MAX_WAIT_TIME_SECONDS=120

rem // commands sent to the CocoaPods daemon are sent over a loopback SSH channel. If no incoming connection happens on this channel
rem // for this delay in seconds, the CocoaPods daemon will consider itself unused and will engage a graceful shutdown procedure.
set AUTOSHUTDOWN_SECONDS=600

rem // end of CocoaPods daemon options
rem //////////////////////////////////

rem // DEV: toggle this option off to let the CocoaPods VM save (commit) its changes to its filesystem backing file (see %COCOAPODS_IMG%).
rem // Toggle this option off then on if you want to, e.g, upgrade the CocoaPods Ruby gems.
rem // BEWARE: this will necessarily change the MD5 checksum of the %COCOAPODS_IMG% file, and you MUST update %VALID_MD5% if you want to
rem // keep using this updated backing file! If you don't do that, the script will download a new copy of the backing file from my server.
set SNAPSHOT_OPTION=-snapshot

rem // for safety, rebuild a minimalistic, default PATH. This is the default PATH for Windows 10.
set PATH=%SystemRoot%\system32;%SystemRoot%;%SystemRoot%\System32\Wbem;%SYSTEMROOT%\System32\WindowsPowerShell\v1.0\

rem // DO NOT switch to the UTF-8 codepage, IT BREAKS THE find.exe UTILITY AND POSSIBLY OTHERS!
rem // chcp 65001 > nul

rem // get the current codepage
for /f "tokens=2 delims=:" %%a in ('chcp') do set ORIGINAL_CP=%%a
set ORIGINAL_CP=%ORIGINAL_CP: =%
set ORIGINAL_CP=%ORIGINAL_CP:.=%

rem // figure out where we are
set SCRIPT_NAME=%~nx0
set SCRIPT_ARGS=%*
set SCRIPT_DIRECTORY=%~dp0

rem // cocoapods filesystem image MD5
set COCOAPODS_IMG=cocoapods-14.3.img
set VALID_MD5=9b5d5ba0cfe0375b7a4592532994db8d

rem // handy shortcut to redirect anything into an environment variable called TEMPVAR
set TEMPVARFILE=%TEMP%\%SCRIPT_NAME%.%RANDOM%%RANDOM%.tmp
set TO_TEMPVAR="%TEMPVARFILE%" 2^> nul ^& set TEMPVAR=^& set /p TEMPVAR=^< "%TEMPVARFILE%" ^& del /f /q "%TEMPVARFILE%" ^> nul 2^>^&1

rem // figure out which TCP port to use to communicate with the CocoaPods daemon. See if it's already running first.
rem // TASKLIST NOTE: on Win32 processes started from a network drive have their image names in CAPITAL LETTERS, hence find /I "..."
tasklist|find /I "cocoapods-daemon.exe" > nul 2>&1 && (
	rem // CocoaPods daemon already running, extract the port it's listening to from its command line
	set LISTEN_PORT=0
	wmic process where caption="cocoapods-daemon.exe" get commandline|find "cocoapods-daemon.exe" > %TO_TEMPVAR%
	set DAEMON_CMDLINE=!TEMPVAR:-^:=^:^:!
	set DAEMON_CMDLINE=!DAEMON_CMDLINE:^:^:=^|!
	for /f "tokens=2 delims=|" %%a in ("!DAEMON_CMDLINE!") do set LISTEN_PORT=%%a
	if "_!LISTEN_PORT!_"=="_0_" echo Error: can't find the TCP port at which the CocoaPods daemon is listening.& goto :exit_error
) || (
	rem // CocoaPods daemon not already started, work out an unused port for it. Start at 15051 and increment until one is free
	set LISTEN_PORT=15050
:choose_tcp_port
	set /a LISTEN_PORT=!LISTEN_PORT!+1>nul
	netstat -a -n|find ":!LISTEN_PORT!">nul && goto :choose_tcp_port
)

rem // set this environment variable to tell our custom SSH/SCP to not bother about insecure key file permissions
set SSH_NOPERMS=true

rem // useful shortcuts
set SSH=.\ssh.exe -p !LISTEN_PORT! -o StrictHostKeyChecking=no -o UserKnownHostsFile=nul -o ServerAliveInterval=60 -i cocoapods.key
set SCP=.\scp.exe -P !LISTEN_PORT! -o StrictHostKeyChecking=no -o UserKnownHostsFile=nul -o ServerAliveInterval=60 -i cocoapods.key -r
set SEND_COMMAND_AS_ROOT=%SSH% -n root@127.0.0.1
set SEND_COMMAND_AS_USER=%SSH% -n cocoapods@127.0.0.1
set IS_DAEMON_RUNNING=tasklist^|find /I "cocoapods-daemon.exe" ^> nul 2^>^&1
set IS_DAEMON_READY=%SEND_COMMAND_AS_ROOT% -o ConnectTimeout=1 -o ConnectionAttempts=1 "if ps ax|grep cocoapods@|grep -v grep > /dev/null; then false; else true; fi" ^> nul 2^>^&1
set WAIT_ONE_SECOND=timeout /t 1 /nobreak ^> nul

rem // position ourselves in the script directory
set ORIGINAL_DIR=%CD%
pushd
cd /d "!SCRIPT_DIRECTORY!"

rem // unless already running, make sure we have enough physical memory to run: !MIN_MEMORY! Mb minimum.
wmic os get FreePhysicalMemory|find /v "FreePhysicalMemory" > %TO_TEMPVAR%
set /a AVAILABLE_MEMORY=!TEMPVAR!/1024
rem // DO NOT CHANGE A SINGLE CHARACTER OF THE LINE BELOW: DELAYED EVALUATION WON'T WORK
%IS_DAEMON_RUNNING% || if %AVAILABLE_MEMORY% LSS %MIN_MEMORY% (
	echo.
	echo It seems your computer doesn't have enough free memory to safely run the
	echo cocoaPods daemon. You need %MIN_MEMORY% Mb free to run the BSD subsystem
	echo and it seems you have only %AVAILABLE_MEMORY% Mb available.
	echo Close a few memory-hungry programs if possible and try again.
	echo.
	goto :exit_error
)

rem // check for proper registration
if not exist cocoapods.key (
	echo.
	echo It appears your version of the CocoaPods daemon hasn't been registered yet.
	echo This service is a separate, optional module of the build environment and
	echo requires separate activation.
	echo.
	echo To use the Windows CocoaPods daemon, please obtain your credentials from
	echo     https://www.pmbaty.com/iosbuildenv/cocoapods/
	echo [InternetShortcut]> activation.url
	echo URL=https://www.pmbaty.com/iosbuildenv/cocoapods/>> activation.url
	start /I "" activation.url
	echo.
	echo When you have received them by e-mail, enter them here:
	set /p ACTIVATION_EMAIL=Activation email: 
	if "_!ACTIVATION_EMAIL!_"=="__" goto :exit_error
	set /p ACTIVATION_CODE=Associated code: 
	if "_!ACTIVATION_CODE!_"=="__" goto :exit_error
	del /f /q activation.url > nul 2>&1
	rem // use powershell's WebClient to download stuff instead of curl as curl has proven unreliable
	rem //.\curl.exe --cacert .\curl-ca-bundle.crt -s -f -o cocoapods.key.tmp https://www.pmbaty.com/iosbuildenv/cocoapods/keys/!ACTIVATION_EMAIL!-!ACTIVATION_CODE! && (
	powershell -Command "(new-object System.Net.WebClient).DownloadFile('https://www.pmbaty.com/iosbuildenv/cocoapods/keys/!ACTIVATION_EMAIL!-!ACTIVATION_CODE!','cocoapods.key.tmp')" 2>nul && (
		echo Bag Attributes> cocoapods.key
		echo     friendlyName: CocoaPods daemon private key for !ACTIVATION_EMAIL!>>cocoapods.key
		type cocoapods.key.tmp >> cocoapods.key
		del cocoapods.key.tmp > nul 2>&1
		echo.
		echo Your key was successfully installed. Thank you^!
	) || (
		echo.
		echo There was an error retrieving your private key. Maybe you misspelled
		echo the activation email or its associated code. Please try again.
		echo.
		goto :exit_error
	)
)

rem // interpret and run the requested command
if "_%1"=="_" goto :usage

if "_%1"=="_start" (
	rem // start up the CocoaPods daemon
	%IS_DAEMON_RUNNING% && (
		echo The CocoaPods daemon is already running.
		goto :exit_error
	)
	if "_!SNAPSHOT_OPTION!_"=="__" goto :boot_vm
	rem // check for presence of the BSD filesystem (test presence and MD5 checksum)
	set SHOULD_DOWNLOAD_COCOAPODS_IMG=false
	if exist "%COCOAPODS_IMG%" (
		certutil -hashfile "%COCOAPODS_IMG%" MD5|find /v "%COCOAPODS_IMG%"|find /v "hashfile" > .hash.txt 2> nul & set CURRENT_MD5=& set /p CURRENT_MD5=< .hash.txt & del /f /q .hash.txt > nul 2>&1
		set CURRENT_MD5=!CURRENT_MD5: =!
		if /i "_!CURRENT_MD5!_" NEQ "_!VALID_MD5!_" (
			taskkill /im cocoapods-daemon.exe > nul 2>&1
			%WAIT_ONE_SECOND%
			del /f /q "%COCOAPODS_IMG%"
			set SHOULD_DOWNLOAD_COCOAPODS_IMG=true
		)
	) else set SHOULD_DOWNLOAD_COCOAPODS_IMG=true
	if "_!SHOULD_DOWNLOAD_COCOAPODS_IMG!_"=="_true_" (
		echo.
		echo Downloading and setting up the BSD UNIX filesystem, please wait...
		echo This step is only done once. Around 160 Mb are being downloaded.
		echo.
	:download_again
		rem // use powershell's WebClient to download stuff instead of curl as curl has proven unreliable
		rem //.\curl.exe --cacert .\curl-ca-bundle.crt -# -f -o "%COCOAPODS_IMG%" "https://www.pmbaty.com/iosbuildenv/cocoapods/%COCOAPODS_IMG%" || (
		powershell -Command "(new-object System.Net.WebClient).DownloadFile('https://www.pmbaty.com/iosbuildenv/cocoapods/%COCOAPODS_IMG%','%COCOAPODS_IMG%')" 2>nul || (
			rem // delete any partially downloaded filesystem
			del /f /q "%COCOAPODS_IMG%" > nul 2>&1
			echo.
			echo It seems the download couldn't complete. Maybe your Internet link is having
			echo problems? Please try again later.
			echo Anyway, the CocoaPods daemon cannot start without this file. Exiting.
			echo.
			goto :exit_error
		)
		rem // verify the downloaded file
		certutil -hashfile "%COCOAPODS_IMG%" MD5|find /v "%COCOAPODS_IMG%"|find /v "hashfile" > .hash.txt 2> nul & set CURRENT_MD5=& set /p CURRENT_MD5=< .hash.txt & del /f /q .hash.txt > nul 2>&1
		set CURRENT_MD5=!CURRENT_MD5: =!
		if /i "_!CURRENT_MD5!_" NEQ "_!VALID_MD5!_" (
			echo.
			echo The MD5 checksum of the downloaded file is wrong. Looks like the download
			echo was corrupt. Trying again...
			echo.
			goto :download_again
		)
		rem // cocoapods filesystem was downloaded and its MD5 is good
		echo.
		echo Download complete.
		echo.
	)
	set SHOULD_DOWNLOAD_COCOAPODS_IMG=
	:boot_vm
	rem // see how many CPU cores we may use. Spare one so that the computer remains responsive.
	rem // NOTE: "wmic" is available on all Windows machines since Windows XP (at least from SP3)
	wmic cpu get NumberOfLogicalProcessors 2>nul | findstr /v "NumberOfLogicalProcessors" > %TO_TEMPVAR%
	rem // remove spaces from value and default to 1 core if we couldn't get the actual number of cores
	if not "_!TEMPVAR!_"=="__" set NUM_CORES=!TEMPVAR: =!
	if "_!NUM_CORES!_"=="__" set NUM_CORES=1
	if !NUM_CORES! gtr 1 set /a NUM_CORES=!NUM_CORES!-1
rem // DO NOT USE ACCELERATION - boot times are dramatically lenghtened. Bug still affecting FreeBSD 12 on qemu. No known fix.
rem //
rem //	rem // ACCELERATION:
rem //	rem // Neither Microsoft Hyper-V nor its predecessor Microsoft Hypervisor Platform are guaranteed to be available on Windows 7.
rem //	rem // To use an ubiquitous hypervisor driver we need to rely on the Intel HAXM driver.
	set ACCEL_FLAG=
rem //	rem // is the VT-D functionality simply not available on this platform ?
rem //	haxm\haxm_check.exe|find /c " -- yes"|find "2" > nul || (
rem //		echo.
rem //		echo INFO: your processor doesn't support hardware accelerated virtualization. The
rem //		echo CocoaPods daemon will run in software emulation mode. This will be very slow^!
rem //	)
rem //	rem // is ***OUR*** HAXM driver NOT installed yet ? if so, install it
rem //	if not exist "haxm\.haxm_install\haxm_install.log" (
rem //		echo.
rem //		echo INFO: it is highly recommended that you install the Intel Hardware Accelerated
rem //		echo eXecution Manager to greatly accelerate the CocoaPods daemon. Please allow the
rem //		echo following installer to do so. It requires Administrator permissions.
rem //		echo.
rem //		rem //powershell Start-Process -Wait -FilePath "haxm\haxm_install.bat" -verb runas
rem //		if     "_!PROCESSOR_ARCHITECTURE!_"=="_AMD64_" set MSI_NAME=hax64.msi
rem //		if not "_!PROCESSOR_ARCHITECTURE!_"=="_AMD64_" set MSI_NAME=hax.msi
rem //		haxm\haxm_install.exe -f haxm\.haxm_install -a /qn /le haxm_install.log MEMSIZETYPE=1 CUSTOMMEMSIZE=0 NOTCHECKVTENABLE=1 && echo Intel Hardware Accelerated eXecution Manager successfully installed.|| (
rem //			echo Intel Hardware Accelerated eXecution Manager installation failed.
rem //			echo.
rem //			echo Install log:
rem //			type haxm\.haxm_install\haxm_install.log
rem //		)
rem //		echo.
rem //	)
rem //	rem // now, is the HAXM driver installed AND available ? if so, use it
rem //	haxm\haxm_check.exe|find /c " -- yes"|find "2" > nul && if exist "%WINDIR%\system32\drivers\IntelHaxm.sys" set ACCEL_FLAG=-accel hax
	rem // have a facility to display a string followed by a carriage return
	echo WScript.StdOut.Write^(WScript.Arguments^(0^) ^& Chr^(13^)^)>echo-r.vbs
	rem // finally, start the CocoaPods daemon
	cscript //nologo echo-r.vbs "Daemon startup: -------------------------------------------------- 0 %% "
	rem // need to use this VBS trick to effectively detach from the Win10 console. Watch your eyes.
	echo CreateObject^("WScript.Shell"^).Run ".\cocoapods-daemon.exe -nodefaults -nographic !ACCEL_FLAG! -cpu max -smp !NUM_CORES! -m !MAX_MEMORY!M -hda %COCOAPODS_IMG% %SNAPSHOT_OPTION% -net nic -net user,hostfwd=tcp::!LISTEN_PORT!-:22", 0 1> daemon-start.vbs & wscript daemon-start.vbs
	%WAIT_ONE_SECOND%
	%IS_DAEMON_RUNNING% || (
		rem // daemon still not kicked up. If we were trying the accelerated version, try the software emulation mode.
		if not "_!ACCEL_FLAG!_"=="__" (
			echo Looks like the CocoaPods daemon refused to start. Trying again with acceleration disabled.
			echo CreateObject^("WScript.Shell"^).Run ".\cocoapods-daemon.exe -nodefaults -nographic -cpu max -smp !NUM_CORES! -m !MAX_MEMORY!M -hda %COCOAPODS_IMG% %SNAPSHOT_OPTION% -net nic -net user,hostfwd=tcp::!LISTEN_PORT!-:22", 0 1> daemon-start.vbs & wscript daemon-start.vbs
			%WAIT_ONE_SECOND%
		)
		%IS_DAEMON_RUNNING% || (echo ERROR: The CocoaPods daemon definitely won't start. Something's wrong with your installation.& del /f /q daemon-start.vbs& goto :exit_error)
	)
	del /f /q daemon-start.vbs
	rem // waits for the CocoaPods daemon to be in a ready state
	set WAITED_SECONDS=0
	if not "_%2"=="_" set /a MAX_STARTUP_TIME_SECONDS=%2>nul
	:start_wait
	%IS_DAEMON_READY% && ( chcp %ORIGINAL_CP% > nul	) || (
		chcp %ORIGINAL_CP% > nul
		rem // the daemon increasingly consumes 1800 Mb memory during startup, which gives us a convenient way to monitor its progress.
		wmic process where name='cocoapods-daemon.exe' get WorkingSetSize|find /v "WorkingSetSize" > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="__" (
			del /f /q echo-r.vbs
			echo Looks like the CocoaPods daemon crashed upon startup. This is an issue.
			echo Please report it to ^<pm@pmbaty.com^> for investigation.
			goto :exit_error
		)
		set /a PERCENTAGE=!TEMPVAR!/18000000> nul
		rem // pure batch progress bar. w00t ^^
		set /a PCTBAR=!PERCENTAGE!/2> nul
		set /p "=Daemon startup: " < nul
		for /l %%i in (1,1,!PCTBAR!) do set /p "=|" < nul
		for /l %%i in (!PCTBAR!,1,48) do set /p "=-" < nul
		cscript //nologo echo-r.vbs "- !PERCENTAGE! %% "
		rem // as the daemon is not ready yet, wait 1 second more and check again
		%WAIT_ONE_SECOND%
		set /a WAITED_SECONDS=!WAITED_SECONDS!+1> nul
		if !WAITED_SECONDS! LSS !MAX_STARTUP_TIME_SECONDS! goto :start_wait
	)
	del /f /q echo-r.vbs
	if !WAITED_SECONDS! GEQ !MAX_STARTUP_TIME_SECONDS! (
		rem // note: the line below extends to 78 characters using trailing spaces ------v
		echo CocoaPods daemon still busy after !MAX_STARTUP_TIME_SECONDS! seconds. Aborting start.     
		taskkill /im cocoapods-daemon.exe > nul 2>&1
		%WAIT_ONE_SECOND%
		goto :exit_error
	)
	rem // note: the line below extends to 78 characters using trailing spaces ------v
	echo Daemon started.                                                              
	rem // at this point the CocoaPods daemon is started and ready to accept commands
	rem // send him the inactivity watchdog to bark around the place. If no opened SSH session in AUTOSHUTDOWN_SECONDS, shutdown the subsystem.
	!SEND_COMMAND_AS_ROOT! "IDLE_COUNT=0; while true; do sleep 10; ps ax|grep 'sshd: '|grep -v '\[listener\]'|grep -v grep > /dev/null && IDLE_COUNT=0 || IDLE_COUNT=$((IDLE_COUNT+1)); test $((IDLE_COUNT*10)) -gt %AUTOSHUTDOWN_SECONDS% && poweroff; done >&- 2>&- <&- &"
	chcp %ORIGINAL_CP% > nul
	goto :exit_success
) else if "_%1"=="_stop" (
	rem // shuts down the CocoaPods daemon
	%IS_DAEMON_RUNNING% || (
		echo The CocoaPods daemon is already stopped.
		goto :exit_error
	)
	echo Stopping CocoaPods daemon...
	!SEND_COMMAND_AS_ROOT! "poweroff" > nul 2>&1
	chcp %ORIGINAL_CP% > nul
	set WAITED_SECONDS=0
	if not "_%2"=="_" set /a MAX_SHUTDOWN_TIME_SECONDS=%2>nul
	:stop_wait
	%IS_DAEMON_RUNNING% && (
		%WAIT_ONE_SECOND%
		set /a WAITED_SECONDS=!WAITED_SECONDS!+1> nul
		if !WAITED_SECONDS! LSS !MAX_SHUTDOWN_TIME_SECONDS! goto :stop_wait
	)
	if !WAITED_SECONDS! GEQ !MAX_SHUTDOWN_TIME_SECONDS! (
		echo CocoaPods daemon still running after !MAX_SHUTDOWN_TIME_SECONDS! seconds. Killing it.
		taskkill /im cocoapods-daemon.exe > nul 2>&1
		%WAIT_ONE_SECOND%
		goto :stop_wait
	)
	goto :exit_success
) else if "_%1"=="_wait" (
	rem // waits for the CocoaPods daemon to be in a ready state
	%IS_DAEMON_RUNNING% || (
		echo The CocoaPods daemon is not running.
		goto :exit_error
	)
	echo Waiting for CocoaPods daemon to be ready...
	set WAITED_SECONDS=0
	if not "_%2"=="_" set /a MAX_WAIT_TIME_SECONDS=%2>nul
	:wait_wait
	%IS_DAEMON_READY% && ( chcp %ORIGINAL_CP% > nul	) || (
		chcp %ORIGINAL_CP% > nul
		%WAIT_ONE_SECOND%
		set /a WAITED_SECONDS=!WAITED_SECONDS!+1> nul
		if !WAITED_SECONDS! LSS !MAX_WAIT_TIME_SECONDS! goto :wait_wait
	)
	if !WAITED_SECONDS! GEQ !MAX_WAIT_TIME_SECONDS! (
		echo CocoaPods daemon still busy after !MAX_WAIT_TIME_SECONDS! seconds. Aborting wait.
		goto :exit_error
	)
	goto :exit_success
) else if "_%1"=="_status" (
	rem // returns whether the CocoaPods daemon is "ready", "busy" or "stopped"
	set STATE=in an unknown state
	%IS_DAEMON_READY% && (
		chcp %ORIGINAL_CP% > nul
		set STATE=ready
	) || (
		chcp %ORIGINAL_CP% > nul
		%IS_DAEMON_RUNNING% && set STATE=busy|| set STATE=stopped
	)
	echo The CocoaPods daemon is !STATE!.
	goto :exit_success
) else if "_%1"=="_reset" (
	rem // undocumented: nuke the CocoaPods database and cache
	%IS_DAEMON_RUNNING% || ( call %0 start || goto :exit_error )
	%IS_DAEMON_READY% && ( chcp %ORIGINAL_CP% > nul ) || ( chcp %ORIGINAL_CP% > nul & ( call %0 wait || goto :exit_error )  )
	rem // clear the working directory on both sides of the pipe, and the CocoaPods caches on the subsystem side
	echo Resetting CocoaPods...
	rmdir /s /q staging & mkdir staging
	!SEND_COMMAND_AS_USER! "killall -q pod && sleep 3; killall -q -KILL pod; rm -rf .cocoapods Library/Caches/CocoaPods staging/*"
	set RET=!errorlevel!
	chcp %ORIGINAL_CP% > nul
	goto :exit
) else if "_%1"=="_gem" (
	rem // sends a RUBY command to the CocoaPods daemon
	%IS_DAEMON_RUNNING% || ( call %0 start || goto :exit_error )
	%IS_DAEMON_READY% && ( chcp %ORIGINAL_CP% > nul ) || ( chcp %ORIGINAL_CP% > nul & ( call %0 wait || goto :exit_error )  )
	rem // make sure no instance of cocoapods is running already
	rem // Also, for a reason unknown to me, /dev/null seems to switch to 600 permissions sometimes. Fix that
	!SEND_COMMAND_AS_ROOT! "killall -q pod && sleep 3; killall -q -KILL pod; chmod 666 /dev/null"
	chcp %ORIGINAL_CP% > nul
	rem // the way is clear
	shift
	echo Sending command: %*
	echo ========================== gem output BEGIN ===========================
	!SEND_COMMAND_AS_ROOT! "%*"
	set RET=!errorlevel!
	chcp %ORIGINAL_CP% > nul
	echo =========================== gem output END ============================
	goto :exit
) else (
	rem // sends a command to the CocoaPods daemon
	%IS_DAEMON_RUNNING% || ( call %0 start || goto :exit_error )
	%IS_DAEMON_READY% && ( chcp %ORIGINAL_CP% > nul ) || ( chcp %ORIGINAL_CP% > nul & ( call %0 wait || goto :exit_error ) )
	rem // clear the working directory on both sides of the pipe
	rem // note that if the ".podspecs_cloning_succeeded" file doesn't exist yet, it's a good idea to wipe the CocoaPods specs cache so as to let it start again cleanly
	rem // Also, for a reason unknown to me, /dev/null seems to switch to 600 permissions sometimes. Fix that
	rmdir /s /q staging 2>nul & mkdir staging
	!SEND_COMMAND_AS_USER! "cd staging && rm -rf *; if [ ! -f ~/.podspecs_cloning_succeeded ]; then killall -q pod && sleep 3; killall -q -KILL pod; rm -rf .cocoapods Library/Caches/CocoaPods; fi"
	!SEND_COMMAND_AS_ROOT! "chmod 666 /dev/null"
	chcp %ORIGINAL_CP% > nul
	rem // populate our working directory with our Podfile and project(s)
	if exist "!ORIGINAL_DIR!\Podfile" (
		echo Preparing staging directory...
		rem // copy the Podfile, and also ANY .podspec and project.pbxproj files from inside the directory where we are, no matter its location, preserving directory structure
		rem // CRUCIAL!!! fix the Podfile so as to use their new CDN instead of GitHub for cloning the master specs repository, which takes ages (and gigabytes of disk space)...
		echo source 'https://cdn.cocoapods.org/'> "staging\Podfile"
		findstr /v "https://github.com/CocoaPods/Specs" "!ORIGINAL_DIR!\Podfile" >> "staging\Podfile"
		(for /f "tokens=1* delims=\" %%A in ('forfiles /p "!ORIGINAL_DIR!" /s /m *.podspec /c "cmd /c echo @relpath"') do for %%F in (^"%%B) do (mkdir "staging\%%~F"& rmdir "staging\%%~F"& copy /b "!ORIGINAL_DIR!\%%~F" "staging\%%~F" >nul)) 2>nul
		(for /f "tokens=1* delims=\" %%A in ('forfiles /p "!ORIGINAL_DIR!" /s /m project.pbxproj /c "cmd /c echo @relpath"') do for %%F in (^"%%B) do (mkdir "staging\%%~F"& rmdir "staging\%%~F"& copy /b "!ORIGINAL_DIR!\%%~F" "staging\%%~F" >nul)) 2>nul
		rem // update CocoaPods daemon working directory from our working directory. Don't you love UNIX tools.
		!SCP! staging cocoapods@127.0.0.1:/Users/cocoapods > nul
		chcp %ORIGINAL_CP% > nul
	)
	rem // the way is clear
	shift
	echo Sending command: "%*"
	echo ========================== CocoaPods output BEGIN ===========================
	!SEND_COMMAND_AS_USER! "cd staging && (%*) && if [ ! -f ~/.podspecs_cloning_succeeded ]; then touch ~/.podspecs_cloning_succeeded; fi"
	set RET=!errorlevel!
	chcp %ORIGINAL_CP% > nul
	echo =========================== CocoaPods output END ============================
	rem // update working directory from the Cocoapods daemon working directory, resolving symlinks (-L)
	if "_!RET!_"=="_0_" (
		if exist "!ORIGINAL_DIR!\Podfile" (
			rem // first step, convert all symlinks into /Users/cocoapods/staging to plain text files containing the symlink path to their target
			echo Converting symlinks...
			!SEND_COMMAND_AS_USER! "cd staging && find . -type l|while read SYMLINK; do TARGET=$(readlink ""${SYMLINK}""); rm ""${SYMLINK}""; echo ""${SYMLINK} -> ${TARGET}""; echo -n ""${TARGET}"" > ""${SYMLINK}""; done"
			chcp %ORIGINAL_CP% > nul
			rem // and now, we can rapatriate the fixed files on the Windows filesystem. Much easier and cleaner than with a half-baked Cygwin-bloated rsync !
			echo Propagating filesystem changes ^(this can take some time, please wait^)...
			!SCP! cocoapods@127.0.0.1:/Users/cocoapods/staging/* "!ORIGINAL_DIR!" > nul
			chcp %ORIGINAL_CP% > nul
			echo Done.
		)
	) else echo NOTICE: this command returned a non-success exit status ^(!RET!^).
	goto :exit
)

:exit_success
set RET=0
goto :exit

:exit_error
set RET=1
goto :exit

:exit

rem // restore the original codepage and return to where we were
chcp %ORIGINAL_CP% > nul
popd
exit /b !RET!
